function crm33000000_Assistente_De_AgendamentoController(CRM_Utilities, ScreenService, templateManager, Messages, General, ZhFilterPreferences) {
    
    
    /**
     * Função para cancelar o gerenciamento do assistente;
     * 
     * @param {object} widget widget de gerenciamento
     * do assistente.
    */
    this.cancelScheduling = function(widget){
        
        var parentWidget = widget.parent.parent;
        
        ScreenService.closeSwipe();
        parentWidget.activate();
        
    };
    /**
     * Função para verificar se o país
     * foi alterado no formulário, se ele foi alterado,
     * a função limpa todos os campos inferiores na 
     * hierarquia de localidade.
     * 
     * @param {object} currentRow currentRow onde está os
     * dados do formulário em caso.
    */
    var selectedCountry;
    this.checkIfTheCountryIsChanged = function(currentRow){
        
      var currentCountrys = currentRow.CDPAIS;

      if(typeof selectedCountry === "undefined"){
          selectedCountry = currentCountrys;
      }else if(selectedCountry !== currentCountrys){
          
          currentRow.NMESTADO = "";
          currentRow.SGESTADO = "";
          currentRow.NMMUNICIPIO = "";
          currentRow.CDMUNICIPIO = "";
          currentRow.NMBAIRRO = "";
          currentRow.NMBAIRROCAN = "";
          currentRow.CDBAIRRO = "";
          
          selectedCountry = currentCountrys;
      }
    };
    
    /**
     * Função para verificar se o estado
     * foi alterado no formulário, se ele foi alterado,
     * a função limpa todos os campos inferiores na 
     * hierarquia de localidade.
     * 
     * @param {object} currentRow currentRow onde está os
     * dados do formulário em caso.
    */
    var selectedState;
    this.checkIfTheStateIsChanged = function(currentRow){
      var currentState = currentRow.SGESTADO;

      if(typeof selectedState === "undefined"){
          selectedState = currentState;
      }else if(selectedState !== currentState){
          
          currentRow.NMMUNICIPIO = "";
          currentRow.CDMUNICIPIO = "";
          currentRow.NMBAIRRO = "";
          currentRow.NMBAIRROCAN = "";
          currentRow.CDBAIRRO = "";
          
          selectedState = currentState;
      }
    };
    
    /**
     * Função para verificar se o município
     * foi alterado no formulário, se ele foi alterado,
     * a função limpa todos os campos inferiores na 
     * hierarquia de localidade.
     * 
     * @param {object} currentRow currentRow onde está os
     * dados do formulário em caso.
    */
    var selectedmunicipality;
    this.checkIfTheCityIsChanged = function(currentRow){
      var currentMunicipality = currentRow.CDMUNICIPIO;

      if(typeof selectedmunicipality === "undefined"){
          selectedmunicipality = currentMunicipality;
      }else if(selectedmunicipality !== currentMunicipality){
          
          currentRow.NMBAIRROCAN = "";
          currentRow.NMBAIRRO = "";
          currentRow.CDBAIRRO = "";
          
          selectedmunicipality = currentMunicipality;
      }
    };
    
    /**
     * Função para definir o filtro dos campos
     * de localidade.
     * 
     * @param {object} field campo (de localidade)
     * em caso.
    */
    this.setAddressFieldFilter = function(field){
        
        CRM_Utilities.setDefaultOrgFilterAndOperatorOrgFilter(field);
        
        CRM_Utilities.setPlaceFilterByPlaceOrigin(field.widget, field);
        
        field.reload();
        
    };
    
    /**
     * Função para abrir o formulário de gerenciamento
     * do assistente(se existir registros filtrados).
     * 
     * @param {objetct} widget widget principal contendo
     * os parceiros que irão ser referenciados nas tarefas.
    */
    this.openFormToGenerateTasks = function(widget){
      
      var tabbedWidget = templateManager.container.getWidget('WDGT_TABBED_GERENCIAMENTO_ASSISTENTE');
      var taskGenerationForm = templateManager.container.getWidget('WDGT_GERENCIAMENTO_ASSISTENETE');
      
      if(widget.dataSource.data.length > 0){
        
        this.setWidgetsEnabledProperty(tabbedWidget.widgets, false);
        
        ScreenService.openSwipe(tabbedWidget);
        ScreenService.closePopup();
        
        taskGenerationForm.enabled = true;
        taskGenerationForm.activate();
        taskGenerationForm.newRow();
        
        this.setGenerateFormDefaultValues(taskGenerationForm);
      
      }else{
        
        ScreenService.confirmMessage(ScreenService.i18n(Messages.NAO_HA_PARCEIROS_FILTRADOR), 'question',
            
            function (success) {
                
                ScreenService.closePopup();
                ZhFilterPreferences.openFilterWidget(widget);
                
            }.bind(this), null
        );
        
      }
      
    };
    
    /**
     * Função para setar os valores iniciais
     * do formulário de gerenciamento do assistente.
     * 
     * @param {object} widget widget do formulário 
     * de gerenciamento do assistente.
    */
    this.setGenerateFormDefaultValues = function(widget){
      
        var currentRow = widget.currentRow;
        
        currentRow.NRPRIORIDADETAREFA = 2;
     
    };
    
    /**
     * Função para setar o filtro dos campos de
     * intervalo da widget principal.
     * 
     * @param {object} widget widget onde se localiza
     * os campos de intervalo.
    **/
    this.setWidgetIntervalFilters = function(widget, args){
        
        this.setEmployeesNumberFilter(widget, args);
        
        this.setBillingValueFilter(widget, args);
        
        delete args.e.preference.FILTER_ROW.NRFUNC_ATE;
        delete args.e.preference.FILTER_ROW.NRFUNC_DE;
        delete args.e.preference.FILTER_ROW.VRFATUPROS_ATE;
        delete args.e.preference.FILTER_ROW.VRFATUPROS_DE;
    };
    
     /**
     * Função para setar o operador de filtragem do campo
     * 
     * @param {object} widget widget onde se localiza
     * os campos.
    */
    this.changeOperatorFilter = function(widget){
        var widgetFilter = widget.dataSourceFilter;
        var diasultevolFilterIndex = CRM_Utilities.getFilterPosition(widgetFilter, "DIASULTEVOL");
        var diasultativconcFilterIndex = CRM_Utilities.getFilterPosition(widgetFilter, "DIASULTATIVCONC");
        
        if (!Util.isUndefined(diasultevolFilterIndex))
            widgetFilter[diasultevolFilterIndex].operator = '>=';
        if (!Util.isUndefined(diasultativconcFilterIndex))    
            widgetFilter[diasultativconcFilterIndex].operator = '>=';
    };
    
    /**
     * Função para setar o filtro do campo
     * de número de funcionários.
     * 
     * @param {object} widget widget onde se localiza
     * o campo de número de funcionários.
    */
    this.setEmployeesNumberFilter = function(widget, args){
        
        var widgetFilter = widget.dataSourceFilter;
        var filterPopupValues = args.e.preference.FILTER_ROW;
        
        var employeesNumberFilterIndex = CRM_Utilities.getFilterPosition(widgetFilter, "NRFUNCIONARIOS");
        
        if(filterPopupValues.NRFUNC_DE > 0 || filterPopupValues.NRFUNC_ATE > 0){
            
            var intervalFilter = this.createNumericalIntervalFilter("NRFUNCIONARIOS", filterPopupValues.NRFUNC_DE, filterPopupValues.NRFUNC_ATE);
            
            if(employeesNumberFilterIndex >= 0){
                widgetFilter[employeesNumberFilterIndex] = intervalFilter;
            }else{
                widgetFilter.push(intervalFilter);
            }
            
        }else if(employeesNumberFilterIndex >= 0){
            widget.dataSourceFilter = CRM_Utilities.deleteFilterObject(widgetFilter, employeesNumberFilterIndex);
            
        }
        
    };
    
    /**
     * Função para setar o filtro do campo
     * de faturamento do parceiro.
     * 
     * @param {object} widget widget onde se localiza
     * o campo de faturamento.
    */
    this.setBillingValueFilter = function(widget, args){
        
        var filterPopupValues = this.getFilterValues(args);
        var widgetFilter = widget.dataSourceFilter;
        
        var intervalInitialValue = CRM_Utilities.removeValueCurrencyMask(filterPopupValues.VRFATUPROS_DE);
        var intervalFinalValue = CRM_Utilities.removeValueCurrencyMask(filterPopupValues.VRFATUPROS_ATE);
        
        var billingValueFilterIndex = CRM_Utilities.getFilterPosition(widgetFilter, "VRFATURAMENTO");
        
        if(intervalInitialValue > 0 || intervalFinalValue > 0){
            
            var intervalFilter = this.createNumericalIntervalFilter("VRFATURAMENTO", filterPopupValues.VRFATUPROS_DE, filterPopupValues.VRFATUPROS_ATE);
            
            if(billingValueFilterIndex >= 0){
                widgetFilter[billingValueFilterIndex] = intervalFilter;
            }else{
                widgetFilter.push(intervalFilter);
            }
            
        }else if(billingValueFilterIndex >= 0){
            widget.dataSourceFilter = CRM_Utilities.deleteFilterObject(widgetFilter, billingValueFilterIndex);
        }
    
    };
    
    /**
     * Função para criar um filtro de intervalor
     * de acordo com o valor inicial e final.
     * 
     * @param {string} fieldReference campo referência
     * para o filtro.
     * 
     * @param {number} initialValue número inicial do 
     * intervalo.
     * 
     * @param {number} finalValue número final do
     * intervalo.
     * 
     * @return {object} filter objeto para ser referenciado
     * no array de objetos(filtros) do campo referência.
    */
    this.createNumericalIntervalFilter = function(fieldReference, initialValue, finalValue){
        
        var filter;
        
        if((initialValue === 0 || initialValue) && (finalValue === 0 || finalValue)){
            filter = {name: fieldReference, operator: 'BETWEEN', value: [initialValue, finalValue]};
        }else if(initialValue === 0 || initialValue){
            filter = {name: fieldReference, operator: '>=', value: initialValue};
        }else if(finalValue === 0 || finalValue){
            filter = {name: fieldReference, operator: '<=', value: finalValue};
        }
        
        return filter;
        
    };
    
    
    /**
     * Função para verificar se os números de um
     * intervalo crescente são válidos (primeiro menor, segundo maior).
     * 
     * @param {object} initialField campo referente ao inicio
     * do intervalo.
     * 
     * @param {object} finalField campo referente ao
     * fim do intervalo.
    */
    this.validateNumericalIntervalBetweenTwoFields = function(initialField, finalField){
        
        var currentRow = initialField.widget.currentRow;
        
        if((currentRow[initialField.name] && currentRow[initialField.name] >= 0) && (currentRow[finalField.name] && currentRow[finalField.name] >= 0)){
            
            if(currentRow[initialField.name] > currentRow[finalField.name]){
                
                currentRow[finalField.name] = undefined;
                ScreenService.showMessage(Messages.INTERVALO_INVALIDO);
                
            }
            
        }   
        
    };
    
    /**
     * Função para verificar se os valores de um
     * intervalo monetário crescente são válidos 
     * (primeiro menor, segundo maior).
     * 
     * @param {object} initialField campo referente ao inicio
     * do intervalo.
     * 
     * @param {object} finalField campo referente ao
     * fim do intervalo.
    */
    this.validateCurrencyIntervalBetweenTwoFields = function(initialField, finalField){
        
        var currentRow = initialField.widget.currentRow;
        
        var initialValue = CRM_Utilities.removeValueCurrencyMask(currentRow[initialField.name]);
        var finalValue = CRM_Utilities.removeValueCurrencyMask(currentRow[finalField.name]);
        
        if((currentRow[initialField.name] && initialValue > 0) && (currentRow[finalField.name] && finalValue > 0)){
            
            if(initialValue > finalValue){
                
                currentRow[finalField.name] = undefined;
                ScreenService.showMessage(Messages.INTERVALO_INVALIDO);
                
            }
            
        }   
        
    };
    
    /**
     * Função para remover os objetos de um filtro
     * referente aos nomes recebidos por parâmetro.
     * 
     * @param {object} fieldFilter array com os objetos
     * referente ao filtro do field em caso.
     * 
     * @param {object} filtersName array com os nomes dos
     * objetos a serem removidos;
    */
    this.removeFiledsFilter = function(widget, filtersName){
        
        var widgetFilter = widget.dataSourceFilter;
        var referenceFilterIndex;
        
        filtersName.map(function(filterName){
            
            referenceFilterIndex = CRM_Utilities.getFilterPosition(widgetFilter, filterName);
            
            if(referenceFilterIndex >= 0){
                widgetFilter = CRM_Utilities.deleteFilterObject(widgetFilter, referenceFilterIndex);
            }
            
        });
        
        widget.dataSourceFilter = widgetFilter;
        
    };
    
    /**
     * Função para retornar os valores do filtro
     * da widget recebida com parâmetro.
     * 
     * @param {object} 'referenceWidget' widget referência;.
     * 
     * @return {object} valores do filtro da widget em caso.
    */
    this.getFilterValues = function(args){
        
        return args.e.preference.FILTER_ROW;
        
    };
    
    /**
     * Define as regras da tela de prospect de acordo 
     * com a revenda do operador logado.
     * 
     * @param {object} widget widget principal da tela de prospect
     * ('WDGT_ASSISTENTE_AGENDAMENTO').
    */
    this.setResaleRules = function(widget) {
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        if(loggedOperatorInfos.IDTPOPERADOR === 'R' && Boolean(loggedOperatorInfos.NRREVENDA)){
            
            var restrictObjects = [];
            
            restrictObjects.push(widget);
            
            var managementWidget = templateManager.container.getWidget("WDGT_GERENCIAMENTO_ASSISTENETE");
            restrictObjects.push(managementWidget.getField("CDOPERADOR"));
            
            CRM_Utilities.setResaleRulesInRestrictObjectsByLoggedOperatorResale(restrictObjects);
            
        }
    };
    
    /**
     * Abri a widget tabbed de gerenciamento do
     * assistente, na aba de tarefas por aniversários.
    */
    this.openTasksByBirthdaySwipe = function(){
        
        var tabbedWidget = templateManager.container.getWidget('WDGT_TABBED_GERENCIAMENTO_ASSISTENTE');
        var tasksByBirthdayWidget = templateManager.container.getWidget('WDGT_ASSISTENTE_ANIVERSARIOS');
        
        ScreenService.openSwipe(tabbedWidget).then(function(){
            
            this.setWidgetsEnabledProperty(tabbedWidget.widgets, false);
            
            ScreenService.closePopup();
            
            tasksByBirthdayWidget.newRow();
            tasksByBirthdayWidget.activate();
            tasksByBirthdayWidget.enabled = true;
            
            this.setTasksByBirthdayDefaultValues(tasksByBirthdayWidget.currentRow);
            
        }.bind(this));
        
    };
    
    /**
     * Seta os valores básicos necessários para gerar tarefas
     * em massa, pelo aniversário do operador responsável.
     * 
     * @param {object} currentRow currentRow que receberá
     * os valores iniciais.
    */
    this.setTasksByBirthdayDefaultValues = function(currentRow){
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.CDOPERADOR = loggedOperatorInfos.CDOPERADOR;
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        
    };
    
    /**
     * Define a propriedade 'enabled' das widgets recebidas como
     * parâmetro.
     * 
     * @param {object} widgets array com as widgets a serem 
     * configuradas.
     * 
     * @param {boolean} isEnabled valor lógico indicando se as
     * widgets ficarão habilitado ou não.
    */
    this.setWidgetsEnabledProperty = function(widgets, isEnabled){
        
        widgets.forEach(function(widget){
            widget.enabled = isEnabled;    
        });
        
    };
    
    /**
     * Realiza uma requição no banco chamando a procedure
     * 'GERA_TAREFA_ANIVER_ANO_ATUAL', responsável por
     * gerar tarefas para os responsável dos parceiros
     * na data de fundação ou aniversário do parceiro.
     * 
     * Ex.: Parceiro: Coca-Cola.
     *      Data Fundação: 12/01/1892.
     *      Responsável: Carlos Eduardo.
     * 
     * Após chamar a procedure, é criada uma tarefa
     * na data de criação da empresa, no ano selecionado.
     * 
     * Data Tarefa: 12/01/2017(ano selecionado).
     * Parceiro; Coca-Cola.
     * Descrição: Desejar feliz aniversário para o
     * parceiro.
     * Responsável pela tarefa: Carlos Eduardo.
     * 
     * @param {object} widget widget de agendamento
     * de tarefas através do aniversário.
    */
    this.generateTasksByBirthday = function(widget) {
        if(widget.isValid()){
            var assistantMainWidget = templateManager.container.getWidget('WDGT_ASSISTENTE_AGENDAMENTO');
            ScreenService.confirmMessage(ScreenService.i18n(Messages.PERGUNTA_GERAR_TAREFAS_ANIVERSARIO), 'question',
            function(success) {
                widget.currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
                var currentRow = widget.currentRow;
                General.callRestEngine("/generateTasksByBirthday", "Row", currentRow).then(function() {
                    ScreenService.showMessage(Messages.TAREFAS_GERADAS_SUCESSO);
                    this.cancelScheduling(widget);
                    assistantMainWidget.activate();
                    
                }.bind(this))
                .catch(function(error) {
                    Messages.CATCH_EXCEPTION_ERROR(error.data.error);
                });
            }.bind(this), null); 
        }
    };
    
    /**
     * Verifica se o ano selecionado pelo usuário é menos que o ano atual.
     * 
     * @param {object} widget widget de agendamento de tarefas através
     * do aniversário.
    */
    this.checkIfSelectedYearIsSmallerThanCurrentYear = function(widget){
        
        var currentRow = widget.currentRow;
        
        if(moment(currentRow.ANO_TAREFAS, 'YYYY').format('YYYY') < moment().format('YYYY')){
            
            ScreenService.showMessage(Messages.ANO_MAIOR_ANO_ATUAL);
            currentRow.ANO_TAREFAS = "";
            
        }
     
    };
    
    /**
     * Abri a widget tabbed de gerenciamento do
     * assistente, na aba de tarefas por intervalo.
    */
    this.openTasksByIntervalSwipe = function(){
        
        var tabbedWidget = templateManager.container.getWidget('WDGT_TABBED_GERENCIAMENTO_ASSISTENTE');
        var tasksByInterval = templateManager.container.getWidget('WDGT_ASSISTENTE_TAREFAS_INTERVALO');
        
        ScreenService.openSwipe(tabbedWidget).then(function(){
            
            this.setWidgetsEnabledProperty(tabbedWidget.widgets, false);
            
            ScreenService.closePopup();
            
            tasksByInterval.newRow();
            tasksByInterval.activate();
            tasksByInterval.enabled = true;
            
            this.setTasksByIntervalDefaultValues(tasksByInterval.currentRow);
            
        }.bind(this));
        
    };
    
    /**
     * Seta os valores básicos necessários para gerar tarefas
     * em massa, pelo intervalo selecionado pelo usuário.
     * 
     * @param {object} currentRow currentRow que receberá
     * os valores iniciais.
    */
    this.setTasksByIntervalDefaultValues = function(currentRow){
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.CDOPERADOR = loggedOperatorInfos.CDOPERADOR;
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        
    };
    
    /**
     * Realiza uma chamada para a procedure 'GERA_TAREFAS_PARC_RELAC_OPER'
     * reponsável por gerar tarefas para os responsáveis
     * de seus respectivos parceiros em um intervalo
     * de tempo.
     * 
     * @param {object} widget widget de agendamento
     * de tarefas por intervalo.
    */
    this.generateTasksByInterval = function(widget){
        var assistantMainWidget = templateManager.container.getWidget('WDGT_ASSISTENTE_AGENDAMENTO');
        if(widget.isValid()){
            
            ScreenService.confirmMessage(ScreenService.i18n(Messages.PERGUNTA_GERAR_TAREFAS_INTERVALO), 'question',
            function (success) {
                widget.currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
                var currentRow = widget.currentRow;
                General.callRestEngine("/generateTasksByInterval", "Row", currentRow).then(function(){
                    ScreenService.showMessage(Messages.TAREFAS_GERADAS_SUCESSO);
                    this.cancelScheduling(widget);
                    assistantMainWidget.activate();
                    
                }.bind(this))
                .catch(function(error){
                    Messages.CATCH_EXCEPTION_ERROR(error.data.error);
                });
            }.bind(this), null);
        }
    };
    
    this.assistAgendaOnEnterOnce = function(widget) {
        var parceiroController = CRM_Utilities.getController('CRM_ParceiroNegocio_Controller');
        var filterRow = {};
        var fieldsToApply = ['IDATIVO'];
        
        CRM_Utilities.openFilterOnEnter(widget, filterRow, fieldsToApply);
        this.setResaleRules(widget);
        parceiroController.alteraEstadoReadonly(widget);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crm33000000_Assistente_De_AgendamentoController', crm33000000_Assistente_De_AgendamentoController);
});