/* global inject expect*/
describe("DevToolsController", function() {

	var devtoolsController;
	var $element;
	var $scope;
	var $rootScope;
	var setCurrentMode = function(mode){
		$scope.lastMode = $scope.currentMode;
		$scope.currentMode = mode;

	};
	var changeCurrentMode = function(mode){
		$scope.setCurrentMode(mode);
	};

	beforeEach(function () {
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.serviceUrl = "../backend/service/index.php";
		module('ZeedhiApp');

		inject( function(_$rootScope_, $controller){
			$rootScope = _$rootScope_;
			$scope = $rootScope.$new();
			$scope.$parent = $rootScope;
			$scope.setCurrentMode = setCurrentMode;
			$scope.changeCurrentMode = changeCurrentMode;

			devtoolsController = $controller("DevToolsController", {
				$scope: $scope,
				$rootScope: $rootScope,
				$element: $element
			});
		});
	});

	it("GetOpenFormImageBasedOnFormVisibility should return 'json-view' if $scope is in Form Mode.", function () {
		$scope.inFormMode = function(){ return true };
		expect(devtoolsController.getOpenFormImageBasedOnFormVisibility()).toEqual('json-view');
	});

	it("GetOpenFormImageBasedOnFormVisibility should return 'form-view' if $scope isn't in Form Mode.", function () {
		$scope.inFormMode = function(){ return false };
		expect(devtoolsController.getOpenFormImageBasedOnFormVisibility()).toEqual('form-view');
	});

	it("GetArrowImageBasedOnTreeVisibility should return 'show-tree' if window.hide is true", function () {
		window.hide = true;
		expect(devtoolsController.getArrowImageBasedOnTreeVisibility()).toEqual('show-tree');
	});

	it("GetArrowImageBasedOnTreeVisibility should return 'hide-tree' if window.hide is false", function () {
		window.hide = false;
		expect(devtoolsController.getArrowImageBasedOnTreeVisibility()).toEqual('hide-tree');
	});

	it("GetTreeTemplateUrlBasedOnEditorMode should return a template from 'developer/tree-item-project.html' if $scope.inFileMode() is true.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/tree-item-project.html';
		$scope.inFileMode = function(){ return true };
		expect(devtoolsController.getTreeTemplateUrlBasedOnEditorMode()).toEqual(urlTemplate);
	});

	it("GetTreeTemplateUrlBasedOnEditorMode should return a template from 'developer/tree-item.html' if $scope.inFileMode() is false.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/tree-item.html';
		$scope.inFileMode = function(){ return false };
		expect(devtoolsController.getTreeTemplateUrlBasedOnEditorMode()).toEqual(urlTemplate);
	});

	it("GetShortcutUrl should return a template from 'developer/zh-search.html' if $scope.isToFindLine() is false.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/zh-search.html';
		$scope.isToFindLine = function(){ return false };
		$scope.isToCreateFile = function(){ return false };
		expect(devtoolsController.getShortcutUrl()).toEqual(urlTemplate);
	});

	it("GetShortcutUrl should return a template from 'zh-create-file.html' if $scope.isToCreateFile() is true.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/zh-create-file.html';
		$scope.isToFindLine = function(){ return false };
		$scope.isToCreateFile = function(){ return true };
		expect(devtoolsController.getShortcutUrl()).toEqual(urlTemplate);
	});

	it("GetShortcutUrl should return a template from 'developer/zh-find-line.html' if $scope.isToFindLine() is true.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/zh-find-line.html';
		$scope.isToFindLine = function(){ return true };
		expect(devtoolsController.getShortcutUrl()).toEqual(urlTemplate);
	});

	it("GetEditorTemplateUrlBasedOnEditorMode should return a template from 'developer/code-editor-json.html' if $scope.inFileMode() and $scope.inFormMode() are false.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/code-editor-json.html';
		$scope.inFileMode = function(){ return false };
		$scope.inFormMode = function(){ return false };
		expect(devtoolsController.getEditorTemplateUrlBasedOnEditorMode()).toEqual(urlTemplate);
	});

	it("GetEditorTemplateUrlBasedOnEditorMode should return a template from 'developer/code-editor-js.html' if $scope.inFileMode() is true.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/code-editor-js.html';
		$scope.inFileMode = function(){ return true };
		$scope.inFormMode = function(){ return false };
		expect(devtoolsController.getEditorTemplateUrlBasedOnEditorMode()).toEqual(urlTemplate);
	});

	it("GetEditorTemplateUrlBasedOnEditorMode should return a template from 'developer/form-editor.html' if $scope.inFormMode() is true.", function () {
		var urlTemplate = '../../dyman/templates/dyman/developer/form-editor.html';
		$scope.inFileMode = function(){ return false };
		$scope.inFormMode = function(){ return true };
		expect(devtoolsController.getEditorTemplateUrlBasedOnEditorMode()).toEqual(urlTemplate);
	});

	it("ChangeCurrentMode should set a current mode in $scope.", function () {
		$scope.currentMode = "js";
		devtoolsController.changeCurrentMode('json');
		expect($scope.currentMode).toBe('json');
	});

	it("ChangeCurrentMode should set a last mode in $scope.", function () {
		$scope.currentMode = "js";
		devtoolsController.changeCurrentMode('json');
		expect($scope.lastMode).toBe('js');
	});

	it("ClearActiveItem should reset a activeItem in $scope.", function () {
		$scope.activeItem = "js";
		devtoolsController.clearActiveItem();
		expect($scope.activeItem).toEqual(undefined);
	});

});
