describe("ContainerDecoratorDeveloper", function() {
	var zhPromise = new ZHPromise(Q);
	var container;
	var containerDecoratorDeveloper;

	beforeEach(function () {
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.serviceUrl = "../backend/service/index.php";
		module('ZeedhiApp');

		container = {
			"footer" : "component/footer.html",
			"height" :  "100%",
			"id" : "572100fdcf7f444c15000308",
			"isVisible" : true,
			"label" : "Login",
			"name" : "login"
		};

		inject(function(templateManager){
			containerDecoratorDeveloper = new ContainerDecoratorDeveloper(zhPromise, templateManager);
		});
	});

	it("ContainerDecoratorDeveloper should apply an enrich in empty object.", function (done) {
		containerDecoratorDeveloper.apply(container.name, {}).then(function(_container) {
			expect(_container.__metadata).toBeDefined();
			expect(_container.__metadata.showInDevTools).toBeDefined();
			expect(_container.showInDevTools).toBeDefined();
			expect(_container.events).toBeDefined();
			expect(_container.events[0].name).toBeDefined();
			expect(_container.events[0].code).toBeDefined();
			done();
		});
	});

	it("ContainerDecoratorDeveloper should apply an enrich in a container.", function (done) {
		containerDecoratorDeveloper.apply(container.name, container).then(function(_container) {
			expect(_container.__metadata).toBeDefined();
			expect(_container.__metadata.showInDevTools).toBeDefined();
			expect(_container.showInDevTools).toBeDefined();
			expect(_container.events).toBeDefined();
			expect(_container.events[0].name).toBeDefined();
			expect(_container.events[0].code).toBeDefined();
			done();
		});
	});

	it("ContainerDecoratorDeveloper shouldn't create a widget with events.", function () {
		expect(container.widgets).toBeUndefined();
		containerDecoratorDeveloper.apply(container.name, container).then(function(_container){
			expect(_container.widgets).toBeUndefined();
			done();
		});
	});

	it("ContainerDecoratorDeveloper should apply an event for each widget.", function () {
		container.widgets = [
			{
				"id": "29208878639523675391378",
				"isVisible" : true,
				"label" : "widgetname",
				"name": "login",
				"widgets" : [{
					"id": "2920887863952367539137878",
					"isVisible" : true,
					"label" : "widgetname",
					"name": "login"
				}]
			}
		];
		containerDecoratorDeveloper.apply(container.name, container).then(function(_container){
			expect(_container.widgets[0].events).toBeDefined();
			expect(_container.widgets[0].events[0].name).toBe("WidgetOnRender");
			expect(_container.widgets[0].events[0].code).toBe("args.owner.redraw()");
			expect(_container.widgets[0].widgets[0].events[0]).toBeDefined();
			expect(_container.widgets[0].widgets[0].events[0].name).toBe("WidgetOnRender");
			expect(_container.widgets[0].widgets[0].events[0].code).toBe("args.owner.redraw()");
			done();
		});
	});

	it("ContainerDecoratorDeveloper should do nothing when containerName isn't defined.", function () {
		var containerName = undefined;
		expect(containerName).toBeUndefined();
		containerDecoratorDeveloper.apply(containerName, container).then(function(_container){
			expect(_container.__metadata).toBeUndefined();
			expect(_container.__metadata.showInDevTools).toBeUndefined();
			expect(_container.showInDevTools).toBeUndefined();
			expect(_container.events).toBeUndefined();
			expect(_container.events[0].name).toBeUndefined();
			expect(_container.events[0].code).toBeUndefined();
			done();
		});
	});

	it("ContainerDecoratorDeveloper should do nothing when container isn't defined.", function () {
		var container = undefined;
		expect(container).toBeUndefined();
		containerDecoratorDeveloper.apply("ContainerName", container).then(function(_container){
			expect(_container.__metadata).toBeUndefined();
			expect(_container.__metadata.showInDevTools).toBeUndefined();
			expect(_container.showInDevTools).toBeUndefined();
			expect(_container.events).toBeUndefined();
			expect(_container.events[0].name).toBeUndefined();
			expect(_container.events[0].code).toBeUndefined();
			done();
		});
	});

	it("ContainerDecoratorDeveloper should do nothing when container isn't an object.", function () {
		var container = "Container";
		expect(typeof container == "object").toBeFalsy();
		containerDecoratorDeveloper.apply("ContainerName", container).then(function(){
			expect(_container.__metadata).toBeUndefined();
			expect(_container.__metadata.showInDevTools).toBeUndefined();
			expect(_container.showInDevTools).toBeUndefined();
			expect(_container.events).toBeUndefined();
			expect(_container.events[0].name).toBeUndefined();
			expect(_container.events[0].code).toBeUndefined();
			done();
		});
	});
});
