var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhTabEditor', function(eventAggregator, ScreenService, templateManager, DevToolsService) {
    return {
        link: function($scope, $element, $attributes, $controller) {
            var developerCommands = ["Ctrl-F", "Command-F", "Ctrl-G", "Command-G", "Ctrl-B", "Command-B"];
            var MODE_JSON = "json";
            var MODE_FORM = "form";
            var MODE_MENU = "menu";
            var MODE_JS = "files";
            ace.config.setDefaultValue("session", "useWorker", false);
            $scope.currentMode = getInitialCurrentMode();
            $scope.currentEditor = null;
            $scope.currentModeClass = null;
            $scope.lastMode = "";

            function getInitialCurrentMode(){
                var mode = MODE_JSON;
                var localDevtools = DevToolsService.getLocalDevToolsFlag();
                var localActiveItem = DevToolsService.getLocalActiveItem();
                if(localDevtools && localActiveItem && localActiveItem.mode){
                    mode = localActiveItem.mode == MODE_FORM ? MODE_JSON : localActiveItem.mode;
                }
                return mode;
            }

            function disableAceCommands(){
                developerCommands.forEach(function(command){
                    $scope.currentEditor.commands.bindKey(command, null);
                });
            }

            function updateCurrentEditor(){
                var codeEditor = inFileMode() ? "code-editor-js" : "code-editor-json";
                var modeClass = inFileMode() ? "ace/mode/javascript" : "ace/mode/json";
                $scope.currentEditor = ace.edit(codeEditor);
                $scope.currentModeClass = ace.require(modeClass).Mode;
                disableAceCommands();
            }

            function actionIsVisibleInContainer(actionName){
                return actionName != "plus" && actionName != "trash";
            }

            function setCurrentMode(mode, devTools){
                if($scope.currentMode != mode){
                    devTools.saveComponentChange();
                    $scope.lastMode = $scope.currentMode;
                    $scope.currentMode = mode;
                }
            }
            $scope.setCurrentMode = setCurrentMode;

            function inFileMode(){
                return $scope.currentMode == MODE_JS;
            }
            $scope.inFileMode = inFileMode;

            function inJsonMode(){
                return $scope.currentMode == MODE_JSON;
            }
            $scope.inJsonMode = inJsonMode;

            function inFormMode(){
                return $scope.currentMode == MODE_FORM;
            }
            $scope.inFormMode = inFormMode;

            function inMenuMode(){
                return $scope.currentMode == MODE_MENU;
            }
            $scope.inMenuMode = inMenuMode;

            function closeMenu(DevToolsCtrl){
                var mode = $scope.lastMode || MODE_JSON;
                setCurrentMode(mode, DevToolsCtrl);
            }
            $scope.closeMenu = closeMenu;

            function initEditor(mode){
                $scope.currentMode = mode || $scope.currentMode;
                if(!inFormMode()){
                    updateCurrentEditor();
                    $scope.currentEditor.setTheme("ace/theme/chaos");
                    $scope.currentEditor.getSession().setTabSize(2);
                    $scope.currentEditor.session.setMode(new $scope.currentModeClass());
                    $scope.currentEditor.$blockScrolling = Infinity;
                    $scope.currentEditor.on('focus', function(){
                        $element.find("textarea").addClass("mousetrap");
                    });
                    $scope.currentEditor.on('change', function(){
                        if($scope.isToSearchText()){
                            $scope.refreshSearch();
                        }
                    });
                    $scope.DevToolsCtrl.changeHeightOfCodeEditor();
                }
            }
            $scope.initEditor = initEditor;

            $scope.getCurrentMode = function(){
                return $scope.currentMode || MODE_JSON;
            };

            $scope.actionIsVisible = function(action, devTools){
                var activeItem = devTools.getActiveTreeItem();
                var condition = false;
                var template = activeItem ? Util.getNestedProperty(activeItem, ["item","template"]): undefined;
                if(template){
                    var isContainer = ~template.indexOf("container");
                    condition = !isContainer || actionIsVisibleInContainer(action);
                }
                return condition;
            };

            $scope.changeFormVisibility = function(devTools){
                var mode = inFormMode() ? MODE_JSON : MODE_FORM;
                setCurrentMode(mode, devTools);
            };

            $scope.$watch("currentEditor", function(){
                var componentPath = $scope.getActiveComponentPath();
                if($scope.inJsonMode() && $scope.lastMode == MODE_FORM && componentPath){
                    var text = $scope.getItemText($scope.getItemMetadataByPath(componentPath));
                    $scope.activeItem.text = text;
                }
            });
        }
    };
});